# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/read_pdz',
                'doc_host': 'https://fligt.github.io',
                'git_url': 'https://github.com/fligt/read_pdz',
                'lib_path': 'read_pdz'},
  'syms': { 'read_pdz.jpg_extractor': { 'read_pdz.jpg_extractor.extract_jpg': ( 'extracting-image-data.html#extract_jpg',
                                                                                'read_pdz/jpg_extractor.py')},
            'read_pdz.legacy': {'read_pdz.legacy.check_pdz_type': ('pdz-archeology.html#check_pdz_type', 'read_pdz/legacy.py')},
            'read_pdz.reader': { 'read_pdz.reader.extract_spectra': ('parsing-bytes.html#extract_spectra', 'read_pdz/reader.py'),
                                 'read_pdz.reader.file_to_bytes': ('parsing-bytes.html#file_to_bytes', 'read_pdz/reader.py'),
                                 'read_pdz.reader.get_block_at': ('parsing-bytes.html#get_block_at', 'read_pdz/reader.py'),
                                 'read_pdz.reader.get_blocks': ('parsing-bytes.html#get_blocks', 'read_pdz/reader.py'),
                                 'read_pdz.reader.get_blocktypes': ('parsing-bytes.html#get_blocktypes', 'read_pdz/reader.py'),
                                 'read_pdz.reader.multiparse': ('parsing-bytes.html#multiparse', 'read_pdz/reader.py'),
                                 'read_pdz.reader.parse': ('parsing-bytes.html#parse', 'read_pdz/reader.py'),
                                 'read_pdz.reader.prefix': ('parsing-bytes.html#prefix', 'read_pdz/reader.py'),
                                 'read_pdz.reader.read_counts': ('parsing-bytes.html#read_counts', 'read_pdz/reader.py'),
                                 'read_pdz.reader.read_strings': ('parsing-bytes.html#read_strings', 'read_pdz/reader.py'),
                                 'read_pdz.reader.read_table': ('parsing-bytes.html#read_table', 'read_pdz/reader.py'),
                                 'read_pdz.reader.skip_bytes': ('parsing-bytes.html#skip_bytes', 'read_pdz/reader.py')}}}
